/*
	-----------------------------------------------------------------------------

	               --- CAEN SpA - Computing Systems Division ---

	-----------------------------------------------------------------------------

	CAENRFIDLib.c

	-----------------------------------------------------------------------------

	Created: Oct 2006

	Version 3.2
  	-----------------------------------------------------------------------------
*/
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include "Protocol.h"
#include "IOLib.h"
#include "CAENRFIDLib.h"
#include "includes_rfid.h"
#include <netinet/in.h>



static short getShort(IOBuffer_t *IOBuffer) {

    short value;
    
    value = ntohs(*(short *) &(IOBuffer->memory)[IOBuffer->rpos]);
    IOBuffer->rpos +=2;
    return value;
}

static int getInt(IOBuffer_t *IOBuffer) {

    int value;
    
    value = ntohl(*(long *) &(IOBuffer->memory)[IOBuffer->rpos]);
    IOBuffer->rpos +=4;
    return value;
}

static long getLong(IOBuffer_t *IOBuffer) {

    long value;
    
    value = ntohl(*(long *) &(IOBuffer->memory)[IOBuffer->rpos]);
    IOBuffer->rpos +=4;
    return value;
}


static char *getString(IOBuffer_t *IOBuffer, int Size) {
    
	int strl;
    char *string = (char *) malloc(Size);
    strncpy(string,&IOBuffer->memory[IOBuffer->rpos],Size);
	strl = strlen(string) + 1;
	IOBuffer->rpos += (Size > strl) ? strl:Size;
    return string;
}

static void InitTagArray(TagArray_t *TagArray) {
    TagArray->size = 0;
    TagArray->Tags = NULL;
}

static void TagArrayAdd(TagArray_t *TagArray, RFIDTag_t *Tag) {
    PList_t *p;
    PList_t *q;
    PList_t *lista;
    
    p = TagArray->Tags;
    q = TagArray->Tags;

    while(p != NULL) {
		q = p;
		p = p->Next;
    }
    lista = (PList_t *) malloc(sizeof(PList_t));
    lista->Data = Tag;
    lista->Next = NULL;
    lista->Prev = q;
	TagArray->size++;
//    printf("tag added=%s\n",lista->Data->id);
    if (TagArray->Tags == NULL) TagArray->Tags = lista; else q->Next = lista;
}

void InitIOBuffer(IOBuffer_t *IOBuffer) {

		IOBuffer->memory = NULL;
		IOBuffer->rpos = 0;
		IOBuffer->size = 0;
		IOBuffer->wpos = 0;
}

void FreeIOBuffer(IOBuffer_t *IOBuffer) {

		if (IOBuffer->memory != NULL) free(IOBuffer->memory);
		IOBuffer->rpos = 0;
		IOBuffer->size = 0;
		IOBuffer->wpos = 0;
}

unsigned short makeHeaderAVP(IOBuffer_t *IOBuffer, unsigned short T11xVer) {

    static unsigned short CommandID = 0;
	unsigned long VendorID = 21336;
	unsigned long hs_l;
	unsigned long hs_s;

		
        IOBuffer->memory = (char *) realloc(IOBuffer->memory,IOBuffer->size + BMF_SIZE);
        if (IOBuffer->memory == NULL) return 0;
		hs_s = htons(T11xVer);
        memcpy(&IOBuffer->memory[IOBuffer->wpos], &hs_s,sizeof(short));
		hs_s = htons(CommandID);
        memcpy(&IOBuffer->memory[IOBuffer->wpos + 2], &hs_s,sizeof(short));
		if (CommandID ==12)
			CommandID=0;
		else
	    	CommandID++;
		hs_l = htonl(VendorID);
        memcpy(&IOBuffer->memory[IOBuffer->wpos + 4], &hs_l,sizeof(long));
		hs_s = htons(0x00);
        memcpy(&IOBuffer->memory[IOBuffer->wpos + 8], &hs_s,sizeof(short));
        IOBuffer->size += BMF_SIZE;
        IOBuffer->wpos += BMF_SIZE;
        return 1;
}

unsigned short addCommandAVP(IOBuffer_t *IOBuffer, unsigned short Command) {


	unsigned long hs_s;

    short AVPSize = BASE_AVP_LENGTH + sizeof(short);
    
    IOBuffer->memory = (char *) realloc(IOBuffer->memory,IOBuffer->size + AVPSize);
    if (IOBuffer->memory == NULL) return 0;
	hs_s = htons(0);
    memcpy(&IOBuffer->memory[IOBuffer->wpos], &hs_s,sizeof(short));
	hs_s = htons(AVPSize);
    memcpy(&IOBuffer->memory[IOBuffer->wpos + 2], &hs_s,sizeof(short)); /* length of the AVP*/
	hs_s = htons(0x01);
    memcpy(&IOBuffer->memory[IOBuffer->wpos + 4], &hs_s,sizeof(short)); 
	hs_s = htons(Command);
    memcpy(&IOBuffer->memory[IOBuffer->wpos + 6], &hs_s,sizeof(short)); 
    IOBuffer->size += AVPSize;
    IOBuffer->wpos += AVPSize;
    return 1;
}


int ShowAVP(IOBuffer_t *IOBuffer) {

int pos;
short currentSize;
short currentType;

    pos = IOBuffer->rpos;
    IOBuffer->rpos += 2;
    currentSize = getShort(IOBuffer);
    currentSize -= 6;
    currentType = getShort(IOBuffer); 
	IOBuffer->rpos = pos;
    return currentType;

}

unsigned short addShortAVP(IOBuffer_t *IOBuffer, void *value, unsigned short type) {

	unsigned long hs_s;
    
    short AVPSize = BASE_AVP_LENGTH + sizeof(short);
    
    IOBuffer->memory = (char *) realloc(IOBuffer->memory,IOBuffer->size + AVPSize);
    if (IOBuffer->memory == NULL) return 0;
	hs_s = htons(0);
    memcpy(&IOBuffer->memory[IOBuffer->wpos], &hs_s,sizeof(short));
	hs_s = htons(AVPSize);
    memcpy(&IOBuffer->memory[IOBuffer->wpos + 2], &hs_s,sizeof(short)); /* length of the AVP*/
	hs_s = htons(type);
    memcpy(&IOBuffer->memory[IOBuffer->wpos + 4], &hs_s,sizeof(short)); 
	hs_s = htons(*(short *)value);
    memcpy(&IOBuffer->memory[IOBuffer->wpos + 6], &hs_s,sizeof(short)); 
    IOBuffer->size += AVPSize;
    IOBuffer->wpos += AVPSize;
    return 1;
}

unsigned short addIntAVP(IOBuffer_t *IOBuffer, void *value, unsigned short type) {

   	unsigned long hs_l;
	unsigned long hs_s;
    
    short AVPSize = BASE_AVP_LENGTH + sizeof(int);

    IOBuffer->memory = (char *) realloc(IOBuffer->memory,IOBuffer->size + AVPSize);
    if (IOBuffer->memory == NULL) return 0;
	hs_s = htons(0);
    memcpy(&IOBuffer->memory[IOBuffer->wpos], &hs_s,sizeof(short));
	hs_s = htons(AVPSize);
    memcpy(&IOBuffer->memory[IOBuffer->wpos + 2], &hs_s,sizeof(short)); /* length of the AVP*/
	hs_s = htons(type);
    memcpy(&IOBuffer->memory[IOBuffer->wpos + 4], &hs_s,sizeof(short)); 
	hs_l = htonl(*(int *)value);
    memcpy(&IOBuffer->memory[IOBuffer->wpos + 6], &hs_l,sizeof(long)); 
    IOBuffer->size += AVPSize;
    IOBuffer->wpos += AVPSize;
    return 1;
}

unsigned short addDataAVP(IOBuffer_t *IOBuffer, void *value, int size, unsigned short type) {

	unsigned long hs_s;
    
    
    short AVPSize = BASE_AVP_LENGTH + size;

    IOBuffer->memory = (char *) realloc(IOBuffer->memory,IOBuffer->size + AVPSize);
    if (IOBuffer->memory == NULL) return 0;
	hs_s = htons(0);
    memcpy(&IOBuffer->memory[IOBuffer->wpos], &hs_s,sizeof(short));
	hs_s = htons(AVPSize);
    memcpy(&IOBuffer->memory[IOBuffer->wpos + 2], &hs_s,sizeof(short)); /* length of the AVP*/
	hs_s = htons(type);
    memcpy(&IOBuffer->memory[IOBuffer->wpos + 4], &hs_s,sizeof(short)); 
    memcpy(&IOBuffer->memory[IOBuffer->wpos + 6],(char *) value,size);
/*	char *buff=(char *)malloc((size+1)*sizeof(char));
	memcpy(buff,&IOBuffer->memory[IOBuffer->wpos + 6],size);
	buff[size]='\0';
	printf("tag=%s\n",buff);*/
    IOBuffer->size += AVPSize;
    IOBuffer->wpos += AVPSize;
    return 1;
}

unsigned short addStringAVP(IOBuffer_t *IOBuffer, void *value, unsigned short type) {

	unsigned long hs_s;
    
    
    short AVPSize;
    int Len;
	char lStr[255];
	memset(lStr,0,255);
    Len = strlen((char *)value) + 1;
	strncpy(lStr,(char *) value,Len);
    if (Len > MAX_CHAR_DESCRIPTION) Len = MAX_CHAR_DESCRIPTION;
    AVPSize = BASE_AVP_LENGTH + Len;
    IOBuffer->memory = (char *) realloc(IOBuffer->memory,IOBuffer->size + AVPSize);
    if (IOBuffer->memory == NULL) return 0;
	hs_s = htons(0);
    memcpy(&IOBuffer->memory[IOBuffer->wpos], &hs_s,sizeof(short));
	hs_s = htons(AVPSize);
    memcpy(&IOBuffer->memory[IOBuffer->wpos + 2], &hs_s,sizeof(short)); /*length of the AVP*/
	hs_s = htons(type);
    memcpy(&IOBuffer->memory[IOBuffer->wpos + 4], &hs_s,sizeof(short)); 
    memcpy(&IOBuffer->memory[IOBuffer->wpos + 6],lStr,Len);
    IOBuffer->size += AVPSize;
    IOBuffer->wpos += AVPSize;
    return 1;
}

void FixPacketLength(IOBuffer_t *IOBuffer) {
	unsigned short length;
	length = htons((short)IOBuffer->size);
	memcpy(&IOBuffer->memory[8],&length,sizeof(short));
}


short GetCommandID(IOBuffer_t *IOBuffer) {
	unsigned short command;
	if (IOBuffer->size < 10) return -1;
	command = ntohs(*(short *) &(IOBuffer->memory)[2]);
	return command;
}

short GetCommand(IOBuffer_t *IOBuffer) {
	unsigned short command;
	if (IOBuffer->size < 18) return -1;
	command = ntohs(*(short *) &(IOBuffer->memory)[16]);
	return command;
}


short GetIDLength_QueryAck(IOBuffer_t *IOBuffer) {
	unsigned short length;
	if (IOBuffer->size < 10) return -1;
	length = ntohs(*(short *) &(IOBuffer->memory)[20]);
	return length;
}
unsigned short checkHeader(IOBuffer_t *IOBuffer,short id) {

    int pos;
    short TxVer;
    short lId;
    int VendorID;
    short PacketLength;
        
        pos = IOBuffer->rpos;
        if ((IOBuffer->size - pos) < 10) return 0;
        TxVer = getShort(IOBuffer);
        lId = getShort(IOBuffer);
        VendorID = getInt(IOBuffer); 
        if ( (TxVer != (short) 0x0001) || 
            (lId != id) ||
            (VendorID != 21336) ) return 0;
        else {
            PacketLength = getShort(IOBuffer);
            return PacketLength;
        }
}

unsigned short checkHeader2(IOBuffer_t *IOBuffer) {

    int pos;
    short TxVer;
    short lId;
    int VendorID;
    short PacketLength;
        
        pos = IOBuffer->rpos;
        if ((IOBuffer->size - pos) < 10) return 0;
        TxVer = getShort(IOBuffer);
        lId = getShort(IOBuffer);
        VendorID = getInt(IOBuffer); 
        if ( (TxVer != (short) 0x0001) || 
            (VendorID != 21336) ) return 0;
        else {
            PacketLength = getShort(IOBuffer);
            return PacketLength;
        }
}

short CheckCommand(IOBuffer_t *IOBuffer, short Command) {

int pos = 0;
short currentSize;
short currentType;
short Result;

    pos = IOBuffer->rpos;
    IOBuffer->rpos += 2;
    currentSize = getShort(IOBuffer);
    currentSize -= 6;
    if (currentSize != 2) return 0;
    currentType = getShort(IOBuffer);
    if (currentType != 1) {
        return 0;
    }
    else {
            if ((IOBuffer->rpos + 2) > IOBuffer->size) return 0;
            Result = getShort(IOBuffer);
    }
    if (Result == Command) return 1; else return 0;
}


short getShortAVP(IOBuffer_t *IOBuffer, short AVPType) {

int pos;
short currentSize;
short currentType;
short Result;

    pos = IOBuffer->rpos;
    IOBuffer->rpos += 2;
    currentSize = getShort(IOBuffer);
    currentSize -= 6;
    if (currentSize != 2) {
        IOBuffer->rpos = pos;
        return CAENRFID_CommunicationError;
    }
    currentType = getShort(IOBuffer); 
    if (currentType != AVPType) {
        return CAENRFID_CommunicationError;
    }
    else {
        if ((IOBuffer->rpos + 2) > IOBuffer->size) return CAENRFID_CommunicationError;
        Result = getShort(IOBuffer);
    }
    return Result;
}     

int getIntAVP(IOBuffer_t *IOBuffer, short AVPType) {

int pos;
short currentSize;
short currentType;

int Result;

    pos = IOBuffer->rpos;
    IOBuffer->rpos += 2;
    currentSize = getShort(IOBuffer);
    currentSize -= 6;
    if (currentSize != 4) {
        IOBuffer->rpos = pos;
        return CAENRFID_CommunicationError;
    }
    currentType = getShort(IOBuffer); 
    if (currentType != AVPType) {
        IOBuffer->rpos = pos;
        return CAENRFID_CommunicationError;
    }
    else {
        if ((IOBuffer->rpos + 4) > IOBuffer->size) return CAENRFID_CommunicationError;
            Result = getInt(IOBuffer);
    }
    return Result;
}    


char *getStringAVP(IOBuffer_t *IOBuffer, short AVPType, int AVPSize){

int pos;
short currentSize;
short currentType;

char *Result = NULL;


    pos = IOBuffer->rpos;
    IOBuffer->rpos += 2;
    currentSize = getShort(IOBuffer);
    currentSize -= 6;
    currentType = getShort(IOBuffer); 
    if (currentType != AVPType) {
        IOBuffer->rpos = pos;
        return NULL;
    }
    else {
        if (currentSize > AVPSize) {
            IOBuffer->rpos = pos;
        return NULL;
        }
        if ((IOBuffer->rpos + currentSize) > IOBuffer->size) return NULL;
            Result = getString(IOBuffer,AVPSize);
        }
    return Result;
}

short getAVPStatus(IOBuffer_t *IOBuffer){

int pos;
short currentSize;
short currentType;

short Result;


    pos = IOBuffer->rpos;
    IOBuffer->rpos += 2;
    currentSize = getShort(IOBuffer);
    currentSize -= 6;
    currentType = getShort(IOBuffer); 
    if (currentType != AVP_RESULT_CODE) {
        IOBuffer->rpos = pos;
        return CAENRFID_CommunicationError;
    }
    else {
        if ((IOBuffer->rpos + currentSize) > IOBuffer->size) return CAENRFID_CommunicationError;
            Result = getShort(IOBuffer);
        }
    return Result;
}

char *getDataAVP_mem(IOBuffer_t *IOBuffer){

int i=0;
int done=0;
short res;
char *Result;
int lint,hint;
	
	Result=(char *)malloc((223-28+2)*sizeof(char));
	Result[0]='\0';
    IOBuffer->rpos+=8;
//	printf("Datas=\n");
	for(i=0;(i<((223-28+1-6)/2) && !done);i++){
		res=getShort(IOBuffer);
		printf("%04x ",res);
		hint=(int)(res/(16*16));
		lint=res%(16*16);
		if (hint!=0){
//			printf("hint=%d;%c     lint=%d;%c\n",hint,hint,lint,lint);
			sprintf(Result,"%s%c",Result,hint);
			if (lint!=0)
				sprintf(Result,"%s%c",Result,lint);
			else
				done=1;
		}
		else
			done=1;
	}
//	printf("Result=%s\n",Result);
	
	return Result;
}


char *getDataAVP_istag(IOBuffer_t *IOBuffer){
int i=0;
short res;
char temp[10];
char *Result;
	
	Result=(char *)malloc((223-28+2)*sizeof(char));
	Result[0]='\0';
    IOBuffer->rpos+=6;
//	printf("Datas=\n");
	for(i=0;i<4;i++){
		res=getShort(IOBuffer);
//		printf("%04x ",res);
		sprintf(temp,"%04x",res);
		if (strlen(temp)>4){
			temp[0]=temp[4];
			temp[1]=temp[5];
			temp[2]=temp[6];
			temp[3]=temp[7];
			temp[4]='\0';
		}
//		printf("hint=%d;%c     lint=%d;%c\n",hint,hint,lint,lint);
		sprintf(Result,"%s%s",Result,temp);

	}
	  //	printf("Result=%s\n",Result);
	
	return Result;
}




char *getDataAVP_Tag(IOBuffer_t *IOBuffer){

int pos;
short currentSize;

char *Result = NULL;
short res;
int i,j;
char temp[10];

    pos = IOBuffer->rpos;
    IOBuffer->rpos += 2;
    currentSize = getShort(IOBuffer);
    currentSize -= 6;
	
	Result=(char *)malloc(100*sizeof(char));
	strcpy(Result,"");
//    printf("Data read: ");
	for(i=0;i<5;i++){
		res=getShort(IOBuffer);
		sprintf(temp,"%04x",res);
		if (strlen(temp)>4){
			temp[0]=temp[4];
			temp[1]=temp[5];
			temp[2]=temp[6];
			temp[3]=temp[7];
			temp[4]='\0';
		}
		sprintf(Result,"%s%s",Result,temp);

//		printf("%s ",temp);
	}
	j=0;
//	printf("  ");
	for (i=0;i<16;i++)
			Result[i]=Result[i+2];
    return Result;
}

struct timeval getTimeStampAVP(IOBuffer_t *IOBuffer, short AVPType) {

int pos;
short currentSize;
short currentType;

struct timeval Result;

	Result.tv_sec = -1;
	Result.tv_usec = -1;
	pos = IOBuffer->rpos;
    IOBuffer->rpos += 2;
    currentSize = getShort(IOBuffer);
    currentSize -= 6;
    if (currentSize != 8) {
        IOBuffer->rpos = pos;
        return Result;
    }
    currentType = getShort(IOBuffer); 
    if (currentType != AVPType) {
        IOBuffer->rpos = pos;
        return Result;
    }
    else {
        if ((IOBuffer->rpos + 8) > IOBuffer->size) return Result;
			
            Result.tv_sec  = getLong(IOBuffer);
			Result.tv_usec = getLong(IOBuffer);
    }
    return Result;

}

TagArray_t getTagsAVP(IOBuffer_t *IOBuffer, char *LSource) {

int pos = 0;
int status = 0;
char *Source= NULL;
char *Antenna = NULL;
struct timeval timev;
short length = 12;
char *ID= NULL;
TagArray_t TagArray;
RFIDTag_t *Tag;
int nbT,i;

InitTagArray(&TagArray);

while(IOBuffer->rpos < IOBuffer->size) {
	if (ShowAVP(IOBuffer) == AVP_SOURCE_NAME) status = 0;
    switch(status) {
        case 0:
                pos = IOBuffer->rpos;
                Source = getStringAVP(IOBuffer,AVP_SOURCE_NAME, MAX_CHAR_DESCRIPTION);
                if (LSource != NULL) {
					if (strncmp(Source,LSource,MAX_CHAR_DESCRIPTION) != 0) {
						if (Source != NULL) free(Source);
						return TagArray;
						}
				}
                status = 1;
            break;
        case 1:
                Antenna = getStringAVP(IOBuffer,AVP_READPOINT_NAME, MAX_READPOINT_NAME);
                if (Antenna == NULL) {
                
                    return TagArray;
                }
                status = 2;
            break;
        case 2:
                timev = getTimeStampAVP(IOBuffer,AVP_TIMESTAMP);
                if ( (timev.tv_sec == -1) && (timev.tv_usec == -1) ) {
                
                    if (Antenna != NULL) free(Antenna);
					if (Source != NULL) free(Source);
                    return TagArray;
                }
                status = 3;
            break;
        case 3:
                length = getShortAVP(IOBuffer,AVP_IDLENGTH);
                if (length == 0) {
                    if (Antenna != NULL) free(Antenna);
					if (Source != NULL) free(Source);
                    return TagArray;
                }
                status = 4;
            break;
        case 4:
				nbT=getnbTags();
				for(i=0;i<nbT;i++){
                	pos = IOBuffer->rpos;
                	ID = getDataAVP_Tag(IOBuffer);

                	if (ID == NULL) {
                    	if (Antenna != NULL) free(Antenna);
						if (Source != NULL) free(Source);
                    	return TagArray;
                	}
                
                	Tag = (RFIDTag_t *)malloc(sizeof(RFIDTag_t));
                	Tag->Length = length;
                	memcpy(Tag->id,ID,MAX_ID_LENGTH);
                	strncpy(Tag->LSource,Source,MAX_CHAR_DESCRIPTION);
                	strcpy(Tag->ReadPoint,Antenna);
                	TagArrayAdd(&TagArray,Tag);
                	if (ID != NULL) {
						free(ID);
						ID = NULL;
					}
					IOBuffer->rpos-=1;
				}
					TagArray.status=CAENRFID_StatusOK;
				 return TagArray;
            break;
		}
	}
	return TagArray;
}



RFIDTag_t TagArrayRem(TagArray_t *TagArray) {

    PList_t *p;
	RFIDTag_t Tag;
    
    p = TagArray->Tags;
    if (p == NULL) {
		Tag.Length = 0;	
		return Tag;
	}
	TagArray->size--;
	Tag.Length = p->Data->Length;
	strncpy(Tag.LSource,p->Data->LSource,MAX_LOGICAL_SOURCE_NAME);
	strncpy(Tag.ReadPoint,p->Data->ReadPoint,MAX_READPOINT_NAME);
	memcpy(Tag.id,p->Data->id,16);
//	printf("tag_id_rem=%s %s\n",p->Data->id,Tag.id);
	free(p->Data);
	if (p->Next) p->Next->Prev = NULL;
	TagArray->Tags = (PList_t *) p->Next;
	free(p);
	return Tag;
}
