#define MAX_CHAR_DESCRIPTION					30
#define MAX_ID_LENGTH							16
#define MAX_ARG_SIZE							50
#define MAX_CLIENT_SUPPORTED					20
#define MAX_READPOINT_NAME						5
#define MAX_LOGICAL_SOURCE_NAME					30
#define BMF_SIZE                                10
#define BASE_AVP_LENGTH                         6

#include <sys/time.h>

typedef struct IOBuffer
{
    char            *memory;
    int			    size;
	int			    rpos;
    int			    wpos;
} IOBuffer_t;

typedef struct RFIDTag
{
    int     Length;    
    int     Type;
    char    id[MAX_ID_LENGTH];
    char    LSource[MAX_CHAR_DESCRIPTION];
    char    ReadPoint[MAX_READPOINT_NAME];
} RFIDTag_t;

typedef struct RFIDNotification
{
    int				Length;    
    char			id[MAX_ID_LENGTH];
    char			LSource[MAX_CHAR_DESCRIPTION];
	struct timeval	TStamp;
	int				Type;
	short			EvtType;
} RFIDNotification_t;

typedef struct PList
{
   RFIDTag_t        *Data;
   struct PList     *Next;
   struct PList     *Prev;
} PList_t;

typedef struct NList
{
   RFIDNotification_t   *Data;
   struct NList			*Next;
   struct NList			*Prev;
} NList_t;

typedef struct TagArray
{
    int             size;
    PList_t         *Tags;
    unsigned        status;
} TagArray_t;

typedef struct NotificationArray
{
    int             size;
    NList_t         *Tags;
    unsigned        status;
} NotificationArray_t;


void InitIOBuffer(IOBuffer_t *IOBuffer);

void FreeIOBuffer(IOBuffer_t *IOBuffer);

unsigned short makeHeaderAVP(IOBuffer_t *IOBuffer, unsigned short T11xVer);

unsigned short addCommandAVP(IOBuffer_t *IOBuffer, unsigned short Command);

unsigned short addShortAVP(IOBuffer_t *IOBuffer, void *value, unsigned short type);

unsigned short addIntAVP(IOBuffer_t *IOBuffer, void *value, unsigned short type);

unsigned short addDataAVP(IOBuffer_t *IOBuffer, void *value, int size, unsigned short type);

unsigned short addStringAVP(IOBuffer_t *IOBuffer, void *value, unsigned short type);

unsigned short checkHeader(IOBuffer_t *IOBuffer,short id);

unsigned short checkHeader2(IOBuffer_t *IOBuffer);

void FixPacketLength(IOBuffer_t *IOBuffer);

short GetCommandID(IOBuffer_t *IOBuffer);

short GetCommand(IOBuffer_t *IOBuffer);

short GetIDLength_QueryAck(IOBuffer_t *IOBuffer);

short CheckCommand(IOBuffer_t *IOBuffer, short Command);

short getShortAVP(IOBuffer_t *IOBuffer, short AVPType);

int getIntAVP(IOBuffer_t *IOBuffer, short AVPType);

char *getStringAVP(IOBuffer_t *IOBuffer, short AVPType, int AVPSize);

short getAVPStatus(IOBuffer_t *IOBuffer);

char *getDataAVP_Tag(IOBuffer_t *IOBuffer);

char *getDataAVP_mem(IOBuffer_t *IOBuffer);

char *getDataAVP_istag(IOBuffer_t *IOBuffer);

TagArray_t getTagsAVP(IOBuffer_t *IOBuffer, char *LSource);

TagArray_t getIDsAVP(IOBuffer_t *IOBuffer, char *LSource);

NotificationArray_t getNotificationAVP(IOBuffer_t *IOBuffer);

NotificationArray_t getNotificationAVPNew(IOBuffer_t *IOBuffer);

RFIDTag_t TagArrayRem(TagArray_t *TagArray);

RFIDNotification_t NotificationArrayRem(NotificationArray_t *TagArray);

NotificationArray_t getReportBufferAVP(IOBuffer_t *IOBuffer);
