#include "includes_rfid.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "list.h"
#include "CAENRFIDLib.h"

#define NBANT 8
CAENRFIDHandle *Handle;
CAENRFIDProtocol *Protocol;
int RS_232_connected=0;

static int antenna_connected[8]={0,0,0,0,0,0,0,0};

/**************************************************************\
 *			RS232_connect
\**************************************************************/
CAENRFIDErrorCodes RS232_connect(char* port, CAENRFIDProtocol proto){
	CAENRFIDErrorCodes error;
	char *ant;
	char *source;
	int i;
	short present;

	Handle=(int *)malloc(sizeof(int));
	Protocol=(CAENRFIDProtocol *)malloc(sizeof(CAENRFIDProtocol));

	printf("CAENRFID_Init\n");
	error = CAENRFID_Init ( RS232, /*protocol : we are using RS232: under Linux, I didn't port the TCP*/
				port, /*Port Number*/
				B115200, /*speed: defined in termios.h*/
				'N', /*no parity*/
				CS8, /*bytesize==8, defined in termios.h*/
				Handle,	/*Handle redirecting to the opened port*/
				Protocol); /*Protocol answered by the RFID-reader*/
	printf("CAENRFID_Init done\n");

	for (i=0;i<NBANT;i++){
	  antenna_connected[i]=0;
	}
	if (error==CAENRFID_StatusOK){
		setRTSLow(*Handle);
		for(i=0;i<4;i++){
			source=(char *)malloc(20*sizeof(char));
			ant=(char *)malloc(20*sizeof(char));
			sprintf(ant,"Ant%d",i);
			sprintf(source,"Source_%d",i);
			error= CAENRFID_GetReadPointInSource(*Handle,ant,source,&present);
			if (error==CAENRFID_StatusOK && present)
					Desactivateantenna(i);
			if (*Protocol!=proto)
				CAENRFID_SetProtocol(*Handle,proto);
			}
		setRTSHigh(*Handle);
		for(i=4;i<8;i++){
			source=(char *)malloc(20*sizeof(char));
			ant=(char *)malloc(20*sizeof(char));
			sprintf(ant,"Ant%d",i-4);
			sprintf(source,"Source_%d",i-4);
			error= CAENRFID_GetReadPointInSource(*Handle,ant,source,&present);
			if (error==CAENRFID_StatusOK && present)
					Desactivateantenna(i);
			if (*Protocol!=proto)
				CAENRFID_SetProtocol(*Handle,proto);
			}
		setRTSLow(*Handle);
		RS_232_connected=1;
		free(source);
		free(ant);
		free(Protocol);

	}
	return error;
}

/**************************************************************\
 *			RS232_disconnect
\**************************************************************/
CAENRFIDErrorCodes RS232_disconnect(){
	reset_External_list();
	reset_Internal_list();
	if (RS_232_connected){
		CAENRFID_End (*Handle);
		RS_232_connected=0;
		free(Handle);
	}

	return CAENRFID_StatusOK;
}



/**************************************************************\
 *			Activateantenna
\**************************************************************/
CAENRFIDErrorCodes Activateantenna(int antenna)
{
	CAENRFIDErrorCodes error,error1;
	CAENRFIDReadPointStatus status_ant;
	char *SourceName;
	char *ReadPoint;
	short present;
	int flag;

	setRTSLow(*Handle);
	if (antenna<0 || antenna>7)
		return CAENRFID_InvalidParam;

	if(!RS_232_connected){
		return CAENRFID_NotConnected;
	}
	if(antenna_connected[antenna])/*just to be sure*/
		Desactivateantenna(antenna);
	
	if (antenna>3){
	  setRTSHigh(*Handle);
	  flag=antenna;
	  antenna-=4;
	}
	else{
	  setRTSLow(*Handle);
	flag=antenna;
	}

	SourceName=(char *)malloc(20*sizeof(char));
		sprintf(SourceName, "Source_%d",antenna);
	ReadPoint=(char *)malloc(20*sizeof(char));
		sprintf(ReadPoint, "Ant%d", antenna);

	if (!antenna_connected[flag])
	  error=CAENRFID_AddReadPoint(*Handle,SourceName,ReadPoint);

	error1=CAENRFID_GetReadPointStatus(*Handle,ReadPoint,&status_ant);
	if (error1==CAENRFID_StatusOK && status_ant>0)
	{
		printf("Erreur Com\n");
			return CAENRFID_CommunicationError;
	}
			
	CAENRFID_GetReadPointInSource(*Handle,ReadPoint,SourceName,&present);
	if(present<0)
	{
		printf("Erreur : not present !!! \n");
			return CAENRFID_CommunicationError;
		
	}

	antenna_connected[flag]=1;
	setRTSLow(*Handle);
	free(SourceName);
	free(ReadPoint);
	return error;
}

/**************************************************************\
 *			Desactivateantenna
\**************************************************************/
CAENRFIDErrorCodes Desactivateantenna(int antenna){
	char *SourceName;
	char *ReadPoint;
	CAENRFIDErrorCodes error;
	int flag;

	setRTSLow(*Handle);
	if (antenna<0 || antenna>7)
		return CAENRFID_InvalidParam;

	if(!RS_232_connected){
		return CAENRFID_NotConnected;
	}

	if(!antenna_connected[antenna]){
			return CAENRFID_StatusOK;
	antenna_connected[antenna]=0;}
	setRTSLow(*Handle);
	if (antenna>3){
	setRTSHigh(*Handle);
	flag=antenna; 
	antenna-=4;
	}else{
	flag=antenna;
	}
	SourceName=(char *)malloc(20*sizeof(char));
		sprintf(SourceName, "Source_%d",antenna);
	ReadPoint=(char *)malloc(20*sizeof(char));
		sprintf(ReadPoint, "Ant%d", antenna);
	if (antenna_connected[flag])
	  error = CAENRFID_RemoveReadPoint(*Handle,SourceName,ReadPoint);
	setRTSLow(*Handle);
	free(SourceName);
	free(ReadPoint);

	return error;
}


/**************************************************************\
 *			get_power
\**************************************************************/
CAENRFIDErrorCodes get_power(unsigned int *powerR){
	CAENRFIDErrorCodes error;
	error = CAENRFID_GetPower (*Handle,powerR);
	return error;
}

/**************************************************************\
 *			set_get_power
\**************************************************************/
CAENRFIDErrorCodes set_get_power(int power,unsigned int *powerR){
	CAENRFIDErrorCodes error;
	if(!RS_232_connected)
		return CAENRFID_NotConnected;

	error = CAENRFID_SetPower(*Handle,power);
	if (error==CAENRFID_StatusOK)
		error = CAENRFID_GetPower (*Handle,powerR);
	return error;
}



/**************************************************************\
 *			acquisition
\**************************************************************/
int acquisition(){
	CAENRFIDErrorCodes error;
	CAENRFIDErrorCodes error_save=CAENRFID_StatusOK;
	CAENRFIDTag *IDList = NULL;
	int i,j,IDNumber;
	char *Tag_ID=(char *)malloc(16*sizeof(char));
	char *source=(char *)malloc(10*sizeof(char));
	
	int nbTags = 0;
		
	reset_External_list();
	reset_Internal_list();
	
	setRTSLow(*Handle);
	for(i=0;i<NBANT;i++)
	{
		if(i==NBANT/2)
		{
			setRTSHigh(*Handle);
		}
		/*if the antenna is not connected, skip this antenna*/
		if(antenna_connected[i]!=1)
		{
			printf("antenna %d not activated\n",i);
			continue;
		}

		sprintf(source,"Source_%d",i%4);
		error = CAENRFID_Inventory(*Handle,source,&IDList,&IDNumber);
		//printf("antenna %d : %d tags\n", i, IDNumber);

		if (error == CAENRFID_StatusOK)
		{
			/*for each Tag identified*/
			for (j =0;j<IDNumber;j++) 
			{
				/*save tag in list*/
				/*get TagId*/
				memcpy(Tag_ID,IDList[j].ID,16);
				Tag_ID[16]='\0';
				/*Store in internal list of tags*/
				nbTags = store_Internal_Tag(Tag_ID,i);
			}/*for j*/
		}/*if StatusOK*/
		else
			error_save=error;
	}/*for i=antennas*/
	
	free(source);
	CAENRFID_FreeTagsMemory(&IDList);
	free(Tag_ID);
	return nbTags;
}



/**************************************************************\
 *			init_RFID
\**************************************************************/
CAENRFIDErrorCodes init_RFID(char* port)
{

        CAENRFIDErrorCodes error;
        unsigned int powerR=-1;
        int i;

       /*Debut d'initialisation..... */
       printf("RS232_connect\n");
        error = RS232_connect(port,ISO18000_6B);
	printf("RS232_connectBefore\n");
        if (error!=CAENRFID_StatusOK) {
               /*si imossible de se connecter au lecteur..*/
                if ((error=RS232_connect(port,ISO18000_6B))!=CAENRFID_StatusOK) {
                        return(error);
                }
        }
	printf("RS232_connect\n");
        /*lecteur connecte*/

        error=set_get_power(7000,&powerR);
        /*si connection perdu*/
        if (error!=CAENRFID_StatusOK || powerR<1000) {
          RS232_disconnect();
                return(error);
        }
        printf("set_get_power\n");

        for (i=0;i<NBANT;i++) 
        {
        	error=Activateantenna(i);
            if (error!=CAENRFID_StatusOK || !antenna_connected[i])
            {
                    printf("Unable to connect antenna %d. Second try ...\n", i);
                    /*deuxieme essai*/
                    error=Activateantenna(i);
                    if (error!=CAENRFID_StatusOK || !antenna_connected[i]) 
                    {
                            printf("Unable to connect antenna %d.\n", i);
                            printf("WARNING!!! Antenna %d not connected continuing activation.\n", i);
//                                      RS232_disconnect();
//                                 return(error);
                    }
                    else
                    {
                            printf("Antenna %d activated\n", i);
                    }
            }
            else 
            {
                    printf("Antenna %d activated\n", i);
            }
        }
        return CAENRFID_StatusOK;
}

/**************************************************************\
 *			close_RFID
\**************************************************************/
CAENRFIDErrorCodes close_RFID()
{
	CAENRFIDErrorCodes error;
	error=RS232_disconnect();
	return(error);
}


